#include <LiquidCrystal.h>
#include <RotaryEncoder.h>          // https://github.com/enjoyneering/RotaryEncoder
#include <EEPROMex.h>               // https://github.com/thijse/Arduino-EEPROMEx
#include "ADF4351.h"

#define RFA             6
#define RFB             7
#define LED             8
#define LD              9
#define ROTARY_PIN_A    2
#define ROTARY_PIN_B    3
#define ROTARY_BUTTON   4
#define BAND_BUTTON     5
#define LCD_RS          1
#define LCD_EN          0
#define LCD_D4          A3
#define LCD_D5          A2
#define LCD_D6          A1
#define LCD_D7          A0
#define VERSION         "v1.00"

const char* const STEPSIZE[]
{
  " 10kHz",
  " 50kHz",
  "100kHz"
};

const char* const MENU[]
{
  "Set output freq.",
  " Low  Edge 70cm ",
  " High Edge 70cm ",
  " Low  Edge 35cm ",
  " High Edge 35cm ",
  " Low  Edge 23cm ",
  " High Edge 23cm ",
  " Low  Edge 13cm ",
  " High Edge 13cm ",
  "   Exit  Menu   "
};

LiquidCrystal lcd(LCD_RS, LCD_EN, LCD_D4, LCD_D5, LCD_D6, LCD_D7);
RotaryEncoder RotaryEncoder(ROTARY_PIN_A, ROTARY_PIN_B, -1);
ADF4351 Frontend;

bool direction;
int16_t position;
long frequency;
unsigned int IF;
unsigned int lowedge;
unsigned int highedge;
byte band;
byte stepsize;
bool menu;
bool menuset;
byte menuitem;

void encoderISR()
{
  RotaryEncoder.readAB();
}

void setup() {
  if (EEPROM.readByte(37) != 1) {
    FactoryDefaults();
  }
  band = EEPROM.readByte(31);
  stepsize = EEPROM.readByte(32);
  IF = EEPROM.readInt(33);
  Frontend.Init(10, 25000000);
  pinMode(RFA, OUTPUT);
  pinMode(RFB, OUTPUT);
  pinMode(LED, OUTPUT);
  pinMode(LD, INPUT);
  pinMode(ROTARY_BUTTON, INPUT_PULLUP);
  pinMode(BAND_BUTTON, INPUT_PULLUP);
  attachInterrupt(digitalPinToInterrupt(ROTARY_PIN_A),  encoderISR,       CHANGE);
  attachInterrupt(digitalPinToInterrupt(ROTARY_PIN_B),  encoderISR,       CHANGE);
  RotaryEncoder.begin();
  lcd.begin(16, 2);
  lcd.setCursor(0, 0);
  lcd.print(" UHF  Converter ");
  lcd.setCursor(0, 1);
  lcd.print("(c) PE5PVB " + String(VERSION));
  delay(1000);
  lcd.clear();
    if (digitalRead(LD) == LOW) {
      lcd.print("   UHF  BOARD");
      lcd.setCursor(0, 1);
      lcd.print(" NOT  DETECTED!");
      exit(0);
    }
  RF();
  ShowFreq();
  ShowIF();
}

void loop() {
  if (position < RotaryEncoder.getPosition()) {
    KeyUp();
  }

  if (position > RotaryEncoder.getPosition()) {
    KeyDown();
  }

  if (digitalRead(ROTARY_BUTTON) == LOW) {
    ButtonPress();
  }

  if (digitalRead(BAND_BUTTON) == LOW) {
    BandPress();
  }
}

void KeyUp() {
  if (menu == false) {
    if (stepsize == 0) {
      frequency += 1;
    } else if (stepsize == 1) {
      frequency += 5;
    } else if (stepsize == 2) {
      frequency += 10;
    }
    if (frequency / 100 > highedge - 1) {
      frequency = lowedge;
      frequency *= 100;
    }
    ShowFreq();
    StoreData();
  } else {
    if (menuset == false) {
      menuitem++;
      if (menuitem > 9) {
        menuitem = 0;
      }
      lcd.setCursor(0, 0);
      lcd.print(MENU[menuitem]);
    } else {
      switch (menuitem) {
        case 0:
          IF += 5;
          if (IF > 10800) {
            IF = 8000;
          }
          MenuShowIF();
          break;

        case 1:
          lowedge++;
          if (lowedge > 439) {
            lowedge = 430;
          }
          MenuShowLow();
          break;

        case 2:
          highedge++;
          if (highedge > 440) {
            highedge = 431;
          }
          MenuShowHigh();
          break;

        case 3:
          lowedge++;
          if (lowedge > 879) {
            lowedge = 860;
          }
          MenuShowLow();
          break;

        case 4:
          highedge++;
          if (highedge > 880) {
            highedge = 861;
          }
          MenuShowHigh();
          break;

        case 5:
          lowedge++;
          if (lowedge > 1309) {
            lowedge = 1280;
          }
          MenuShowLow();
          break;

        case 6:
          highedge++;
          if (highedge > 1310) {
            highedge = 1280;
          }
          MenuShowHigh();
          break;

        case 7:
          lowedge++;
          if (lowedge > 2399) {
            lowedge = 2300;
          }
          MenuShowLow();
          break;

        case 8:
          highedge++;
          if (highedge > 2400) {
            highedge = 2301;
          }
          MenuShowHigh();
          break;
      }
    }
  }
  while (digitalRead(ROTARY_PIN_A) == LOW || digitalRead(ROTARY_PIN_B) == LOW) {
  }
  position = RotaryEncoder.getPosition();
}

void KeyDown() {
  if (menu == false) {
    if (stepsize == 0) {
      frequency -= 1;
    } else if (stepsize == 1) {
      frequency -= 5;
    } else if (stepsize == 2) {
      frequency -= 10;
    }
    if (frequency / 100 < lowedge) {
      frequency = highedge;
      frequency *= 100;
    }
    ShowFreq();
    StoreData();
  } else {
    if (menuset == false) {
      menuitem--;
      if (menuitem > 9) {
        menuitem = 9;
      }
      lcd.setCursor(0, 0);
      lcd.print(MENU[menuitem]);
    } else {
      switch (menuitem) {
        case 0:
          IF -= 5;
          if (IF < 8000) {
            IF = 10800;
          }
          MenuShowIF();
          break;

        case 1:
          lowedge--;
          if (lowedge < 430) {
            lowedge = 439;
          }
          MenuShowLow();
          break;

        case 2:
          highedge--;
          if (highedge < 431) {
            highedge = 440;
          }
          MenuShowHigh();
          break;

        case 3:
          lowedge--;
          if (lowedge < 860) {
            lowedge = 879;
          }
          MenuShowLow();
          break;

        case 4:
          highedge--;
          if (highedge < 861) {
            highedge = 880;
          }
          MenuShowHigh();
          break;

        case 5:
          lowedge--;
          if (lowedge < 1280) {
            lowedge = 1309;
          }
          MenuShowLow();
          break;

        case 6:
          highedge--;
          if (highedge < 1281) {
            highedge = 1310;
          }
          MenuShowHigh();
          break;

        case 7:
          lowedge--;
          if (lowedge < 2300) {
            lowedge = 2399;
          }
          MenuShowLow();
          break;

        case 8:
          highedge--;
          if (highedge < 2301) {
            highedge = 2400;
          }
          MenuShowHigh();
          break;

      }
    }
  }
  while (digitalRead(ROTARY_PIN_A) == LOW || digitalRead(ROTARY_PIN_B) == LOW) {
  }
  position = RotaryEncoder.getPosition();
}

void ButtonPress() {
  const unsigned long timer = millis();
  while (digitalRead(ROTARY_BUTTON) == LOW && millis() - timer < 2000) {}
  if (millis() - timer < 2000) {
    if (menu == false) {
      stepsize++;
      if (stepsize > 2) {
        stepsize = 0;
      }
      ShowStepSize();
    } else {
      if (menuset == false) {
        ShowMenuItem();
      } else {
        ExitMenuItem();
      }
    }
  } else {
    menu = true;
    menuitem = 0;
    lcd.clear();
    lcd.print(MENU[menuitem]);
  }
  while (digitalRead(ROTARY_BUTTON) == LOW) {
    delay(50);
  }
}

void BandPress() {
  if (menu == false) {
    band ++;
    if (band > 4) {
      band = 1;
    }
    RF();
    ShowFreq();
    StoreData();
    while (digitalRead(BAND_BUTTON) == LOW) {
      delay(50);
    }
  }
}

void ShowFreq() {
  ShowStepSize();
  lcd.home();
  if (frequency < 100000) {
    lcd.print(" ");
  }
  lcd.print(frequency / 100);
  lcd.print(".");
  if (frequency % 100 < 10) {
    lcd.print("0");
  }
  lcd.print(frequency % 100);
  lcd.print("MHz");
  Frontend.SetFreq(frequency - IF);
}

void ShowIF() {
  lcd.setCursor(0, 1);
  if (IF < 10000) {
    lcd.print(" ");
  }
  lcd.print(" ");
  lcd.print(IF / 100);
  lcd.print(".");
  if (IF % 100 < 10) {
    lcd.print("0");
  }
  lcd.print(IF % 100);
  lcd.print("MHz");
}

void RF() {
  lcd.setCursor(12, 0);
  if (band == 1) {
    frequency = EEPROM.readLong(0);
    lowedge = EEPROM.readInt(38);
    highedge = EEPROM.readInt(42);
    digitalWrite(RFA, LOW);
    digitalWrite(RFB, LOW);
    lcd.print("70cm");
  } else if (band == 2) {
    frequency = EEPROM.readLong(7);
    lowedge = EEPROM.readInt(46);
    highedge = EEPROM.readInt(50);
    digitalWrite(RFA, HIGH);
    digitalWrite(RFB, LOW);
    lcd.print("35cm");
  } else if (band == 3) {
    frequency = EEPROM.readLong(15);
    lowedge = EEPROM.readInt(54);
    highedge = EEPROM.readInt(58);
    digitalWrite(RFA, LOW);
    digitalWrite(RFB, HIGH);
    lcd.print("23cm");
  } else if (band == 4) {
    frequency = EEPROM.readLong(23);
    lowedge = EEPROM.readInt(62);
    highedge = EEPROM.readInt(66);
    digitalWrite(RFA, HIGH);
    digitalWrite(RFB, HIGH);
    lcd.print("13cm");
  }
}

void ShowStepSize() {
  lcd.setCursor(10, 1);
  lcd.print(STEPSIZE[stepsize]);
}

void FactoryDefaults() {
  EEPROM.writeLong(0, 43350);       // Band 1
  EEPROM.writeLong(7, 87450);       // Band 2
  EEPROM.writeLong(15, 129950);     // Band 3
  EEPROM.writeLong(23, 232550);     // Band 4
  EEPROM.writeByte(31, 1);          // Band keuze
  EEPROM.writeByte(32, 1);          // Stepsize
  EEPROM.writeInt(33, 10000);       // IF Frequentie
  EEPROM.writeByte(37, 1);          // Versie controle
  EEPROM.writeInt(38, 430);         // Low edge band 1
  EEPROM.writeInt(42, 440);         // High edge band 1
  EEPROM.writeInt(46, 870);         // Low edge band 2
  EEPROM.writeInt(50, 880);         // High edge band 2
  EEPROM.writeInt(54, 1240);        // Low edge band 3
  EEPROM.writeInt(58, 1310);        // High edge band 3
  EEPROM.writeInt(62, 2320);        // Low edge band 4
  EEPROM.writeInt(66, 2400);        // High edge band 4
}

void StoreData() {
  if (band == 1) {
    EEPROM.writeLong(0, frequency);
  } else if (band == 2) {
    EEPROM.writeLong(7, frequency);
  } else if (band == 3) {
    EEPROM.writeLong(15, frequency);
  } else if (band == 4) {
    EEPROM.writeLong(23, frequency);
  }
  EEPROM.writeByte(31, band);
  EEPROM.writeByte(32, stepsize);
}

void ShowMenuItem() {
  menuset = true;
  lcd.setCursor(0, 1);
  lcd.print("                ");
  switch (menuitem) {
    case 0:
      MenuShowIF();
      break;

    case 1:
      lowedge = EEPROM.readInt(38);
      MenuShowLow();
      break;

    case 2:
      highedge = EEPROM.readInt(42);
      MenuShowHigh();
      break;

    case 3:
      lowedge = EEPROM.readInt(46);
      MenuShowLow();
      break;

    case 4:
      highedge = EEPROM.readInt(50);
      MenuShowHigh();
      break;

    case 5:
      lowedge = EEPROM.readInt(54);
      MenuShowLow();
      break;

    case 6:
      highedge = EEPROM.readInt(58);
      MenuShowHigh();
      break;

    case 7:
      lowedge = EEPROM.readInt(62);
      MenuShowLow();
      break;

    case 8:
      highedge = EEPROM.readInt(66);
      MenuShowHigh();
      break;

    case 9:
      menu = false;
      menuset = false;
      lcd.clear();
      RF();
      ShowFreq();
      ShowIF();
      break;
  }
}

void ExitMenuItem() {
  menuset = false;
  lcd.setCursor(0, 1);
  lcd.print("                ");
  switch (menuitem) {
    case 0:
      EEPROM.writeInt(33, IF);
      break;

    case 1:
      EEPROM.writeInt(38, lowedge);
      break;

    case 2:
      EEPROM.writeInt(42, highedge);
      break;

    case 3:
      EEPROM.writeInt(46, lowedge);
      break;

    case 4:
      EEPROM.writeInt(50, highedge);
      break;

    case 5:
      EEPROM.writeInt(54, lowedge);
      break;

    case 6:
      EEPROM.writeInt(58, highedge);
      break;

    case 7:
      EEPROM.writeInt(62, lowedge);
      break;

    case 8:
      EEPROM.writeInt(66, highedge);
      break;
  }
}

void MenuShowIF() {
  lcd.setCursor(3, 1);
  if (IF < 10000) {
    lcd.print(" ");
  }
  lcd.print(IF / 100);
  lcd.print(".");
  if (IF % 100 < 10) {
    lcd.print("0");
  }
  lcd.print(IF % 100);
  lcd.print(" MHz");
}

void MenuShowLow() {
  if (lowedge > 1000) {
    lcd.setCursor(4, 1);
  } else {
    lcd.setCursor(5, 1);
  }
  lcd.print(lowedge);
  lcd.print(" MHz");
}

void MenuShowHigh() {
  if (highedge > 1000) {
    lcd.setCursor(4, 1);
  } else {
    lcd.setCursor(5, 1);
  }
  lcd.print(highedge);
  lcd.print(" MHz");
}
